from flask import current_app
from . import docker_client
import docker
import logging
import os
import shutil


def create_build_corpus_service(corpus):
    corpus_dir = os.path.join(current_app.config['DATA_DIR'],
                              str(corpus.user_id),
                              'corpora',
                              str(corpus.id))
    corpus_data_dir = os.path.join(corpus_dir, 'data')
    corpus_file = os.path.join(corpus_dir, 'merged', 'corpus.vrt')
    corpus_registry_dir = os.path.join(corpus_dir, 'registry')
    if os.path.exists(corpus_data_dir):
        shutil.rmtree(corpus_data_dir)
    if os.path.exists(corpus_registry_dir):
        shutil.rmtree(corpus_registry_dir)
    os.mkdir(corpus_data_dir)
    os.mkdir(corpus_registry_dir)
    service_args = {
        'command': 'docker-entrypoint.sh build-corpus',
        'constraints': ['node.role==worker'],
        'labels': {'origin': 'nopaque',
                   'type': 'corpus.prepare',
                   'corpus_id': str(corpus.id)},
        'mounts': [corpus_file + ':/root/files/corpus.vrt:ro',
                   corpus_data_dir + ':/corpora/data:rw',
                   corpus_registry_dir + ':/usr/local/share/cwb/registry:rw'],
        'name': 'build-corpus_{}'.format(corpus.id),
        'restart_policy': docker.types.RestartPolicy()
    }
    service_image = \
        'gitlab.ub.uni-bielefeld.de:4567/sfb1288inf/cqpserver:latest'
    try:
        docker_client.services.create(service_image, **service_args)
    except docker.errors.APIError as e:
        logging.error('create_build_corpus_service({}): '.format(corpus.id)
                      + '{} (status: {} -> failed)'.format(e, corpus.status))
        corpus.status = 'failed'
    else:
        corpus.status = 'queued'
    finally:
        # TODO: send email
        pass


def checkout_build_corpus_service(corpus):
    service_name = 'build-corpus_{}'.format(corpus.id)
    try:
        service = docker_client.services.get(service_name)
    except docker.errors.NotFound as e:
        logging.error('checkout_build_corpus_service({}):'.format(corpus.id)
                      + ' {} (stauts: {} -> failed)'.format(e, corpus.status))
        corpus.status = 'failed'
    # TODO: handle docker.errors.APIError and docker.errors.InvalidVersion
    else:
        service_tasks = service.tasks()
        if not service_tasks:
            return
        task_state = service_tasks[0].get('Status').get('State')
        if corpus.status == 'queued' and task_state != 'pending':
            corpus.status = 'running'
        elif corpus.status == 'running' and task_state == 'complete':
            service.remove()
            corpus.status = 'prepared'
        elif corpus.status == 'running' and task_state == 'failed':
            service.remove()
            corpus.status = task_state
    finally:
        # TODO: send email
        pass


def create_cqpserver_container(corpus):
    corpus_dir = os.path.join(current_app.config['DATA_DIR'],
                              str(corpus.user_id),
                              'corpora',
                              str(corpus.id))
    corpus_data_dir = os.path.join(corpus_dir, 'data')
    corpus_registry_dir = os.path.join(corpus_dir, 'registry')
    container_args = {
        'command': 'cqpserver',
        'detach': True,
        'volumes': [corpus_data_dir + ':/corpora/data:rw',
                    corpus_registry_dir + ':/usr/local/share/cwb/registry:rw'],
        'name': 'cqpserver_{}'.format(corpus.id),
        'network': 'nopaque_default'
    }
    container_image = \
        'gitlab.ub.uni-bielefeld.de:4567/sfb1288inf/cqpserver:latest'
    try:
        container = docker_client.containers.get(container_args['name'])
    except docker.errors.NotFound:
        pass
    except docker.errors.DockerException:
        return
    else:
        container.remove(force=True)
    try:
        docker_client.containers.run(container_image, **container_args)
    except docker.errors.DockerException:
        return
    else:
        corpus.status = 'analysing'


def remove_cqpserver_container(corpus):
    container_name = 'cqpserver_{}'.format(corpus.id)
    try:
        container = docker_client.containers.get(container_name)
    except docker.errors.NotFound:
        pass
    except docker.errors.DockerException:
        return
    else:
        container.remove(force=True)
    corpus.status = 'prepared'
